@@ -1,4 +1,3 @@
-#
 use v5.10.1;
 use strict;
 use warnings;
@@ -59,9 +58,6 @@ sub message {
       );
     my @in;
     for my $ctx (@contexts) {
-        if ( my $newfile = $ctx->{including_file} || $ctx->{template_file} ) {
-            $file = $newfile;
-        }
         if ( $ctx->{current_macro} ) {
             push @in,
                 "... in macro "
@@ -69,6 +65,9 @@ sub message {
               . $ctx->{current_param}
               . ") at $file";
         }
+        if ( my $newfile = $ctx->{including_file} || $ctx->{template_file} ) {
+            $file = $newfile;
+        }
     }
 
     for my $msg ( reverse @in ) {
@@ -93,9 +92,12 @@ my %preexpand = map { $_ => 1 } qw<
   insert insert_capture insert_filelist
   expand template ctx_template script ctx_script
   sp_escape nl_escape c_escape fixup uc lc abs2rel
-  shquot mkquot chomp if
+  shquot mkquot chomp if bpv bpm bsv bsm echo
+  use_prereqs
 >;
 
+my %receipe_macro;
+
 # Hash of externally registered macros.
 my %external;
 
@@ -125,8 +127,9 @@ sub register_macro {
     $self->throw("Bad macro name '$name'") unless $name && $name =~ /^\w+$/;
     $self->throw("Macro sub isn't a code ref") unless ref($sub) eq 'CODE';
 
-    $external{$name}  = $sub;
-    $preexpand{$name} = !!$params{preexpand};
+    $external{$name}      = $sub;
+    $preexpand{$name}     = !!$params{preexpand};
+    $receipe_macro{$name} = !!$params{in_receipe};
 }
 
 sub cfg { $_[0]->{config_obj} }
@@ -204,7 +207,17 @@ sub execute {
     }
 
     my $out;
-    eval { $out = $self->$method($param); };
+    eval {
+        my $msub = sub {
+            $out = $self->$method($param);
+        };
+        if ( $receipe_macro{$macro} ) {
+            $self->in_receipe_context($msub);
+        }
+        else {
+            $msub->();
+        }
+    };
     if ($@) {
         $self->throw( $@, callstack => longmess("") );
     }
@@ -251,13 +264,13 @@ sub _expand {
                   | (?<macro>
                         (?<msym> (?: @@ | @))
                         (?:
-                            (?<macro_var> \w [:\w]* )
+                            (?<macro_var> \w [:\w\-]* )
                           | (?: 
                               (?: 
                                   (?<mfunc_noexp> ! )
                                 | (?<mfunc_if_can> \? )
                               )* 
-                              (?<macro_func> \w [:\w]* )
+                              (?<macro_func> \w [:\w\-]* )
                               (?>
                                 \(
                                   (?<mparam>
@@ -265,7 +278,7 @@ sub _expand {
                                         (?2)
                                       | [^\)]
                                       | \) (?! \k<msym> )
-                                      | (?(?{ $+{msym} eq '@' }) \z (?{ $self->throw( "Can't find closing \)$+{msym} for macro '$+{macro_func}' following «" . $last_text . "»" ) }))
+                                      | (?(?{ $+{msym} eq '@' }) \z (?{ $self->throw( "Can't find closing \)$+{msym} for macro '$+{macro_func}' following <<" . $last_text . ">>" ) }))
                                     )*
                                   )
                                 \)
@@ -412,6 +425,59 @@ sub not_in_context {
     }
 }
 
+sub is_in_context {
+    my $self = shift;
+    my ( $ctx_name, $ctx_prop ) = @_;
+    my $cfg = $self->{config_obj};
+    unless ( $cfg->prop($ctx_prop) ) {
+        $self->throw("Required '$ctx_name' context not found.");
+    }
+}
+
+# Execute callback in a custom context.
+sub do_in_context {
+    my $self  = shift;
+    my $cb    = shift;
+    my %props = @_;
+
+    my $ctx = $props{ctx}
+      or $self->throw("do_in_context requires 'ctx' named parameter");
+    if ( ref($ctx) ) {
+        $self->throw(
+            "do_in_context requires 'ctx' named parameter to be a hash")
+          unless ref($ctx) eq 'HASH';
+    }
+    else {
+        $ctx = { $ctx => 1 };
+    }
+    my $configs = ( $props{config} && [ $props{config} ] ) || $props{configs};
+    $ctx->{configs} = $configs;
+    my $s = $self->cfg->push_ctx($ctx);
+    return $cb->();
+}
+
+# Execute a callback in receipe context
+sub in_receipe_context {
+    my $self   = shift;
+    my $cb     = shift;
+    my %config = (
+        prereqs => $self->cfg->cfg('make_all_prereq'),
+        @_
+    );
+    $self->do_in_context(
+        $cb,
+        ctx    => '.make_receipe',
+        config => \%config
+    );
+}
+
+# Set a config variable in receipe context.
+sub set_in_receipe {
+    my $self = shift;
+    my ($var, $val) = @_;
+    $self->cfg->set($var, $val, in_ctx => '.make_receipe');
+}
+
 sub backends_iterate {
     my $self = shift;
     my $cfg  = $self->{config_obj};
@@ -421,14 +487,21 @@ sub backends_iterate {
     my $cb = shift;
 
     for my $be ( $cfg->active_backends ) {
+        my $babbr  = $cfg->backend_abbr($be);
         my %config = (
             ctx_subdir     => $be,
             backend_subdir => $be,
             backend        => $be,
-            backend_abbr   => $cfg->backend_abbr($be),
-            backend_prefix => $cfg->backend_abbr($be),
+            backend_abbr   => $babbr,
+            backend_prefix => $babbr,
+            bp             => uc($babbr) . "_",
+            bext           => $cfg->backend_ext($be),
+            btarget        => $cfg->backend_target($be),
         );
+        my %iprops = %{ $cfg->{impls}{$be} };
+        delete $iprops{config};
         my $be_ctx = {
+            %iprops,
             backend => $be,
             configs => [ $cfg->{impls}{$be}{config}, \%config ],
         };
@@ -626,7 +699,7 @@ sub _m_nl_escape {
 # Escaping for c string literals.
 sub _m_c_escape {
     my $self = shift;
-    my $str = shift;
+    my $str  = shift;
     $str =~ s{\\}{\\\\}sg;
     $str =~ s{"}{\\"}sg;
     return $str;
@@ -732,6 +805,30 @@ sub _m_q {
     return $q . shift . $q;
 }
 
+# echo(str)
+# Produces echo command for Makefile. Takes special care of Windows oddities.
+sub _m_echo {
+    my $self = shift;
+    my $text = shift;
+    return '@echo '
+      . (
+          $self->cfg->is_win
+        ? $text
+        : $self->cfg->shell_quote_filename($text)
+      );
+}
+
+# use_prereqs(str)
+# Records prerequisites to be used to buid current target in a makefile rule
+# Allows a target to depend on more prerequisites than directly used to build
+# it.
+sub _m_use_prereqs {
+    my $self = shift;
+    my $text = shift;
+    $self->cfg->set( 'prereqs', $text, in_ctx => '.make_receipe' );
+    return $text;
+}
+
 # abs2rel(file1 file2)
 # Converts absolute file path into relative to @base_dir@
 sub _m_abs2rel {
@@ -861,8 +958,13 @@ sub _m_if {
     my $text = shift;
 
     my $out = "";
-    if ( $text =~ s/^(?<cond>\S+)\s+(.*)/$2/ ) {
-        my $cond    = $+{cond};
+    if ( $text =~ /^(?<cond>\S+)(?<ws>\s)(?<text>.*)/s ) {
+        my $cond = $+{cond};
+        my $ws   = $+{ws};
+
+        # Prepend back any non-space whitespace to the text. Mostly useful for
+        # preserving \t in makefiles.
+        $text = ( $ws eq ' ' ? '' : $ws ) . $+{text};
         my $matches = 0;
         if ( $cond =~ /^(?<var>\w(?:\w|:\w)*)(?:(?<op>[=\!]=)(?<val>.*))?$/ ) {
             if ( $+{op} ) {
@@ -870,8 +972,7 @@ sub _m_if {
                 my $var      = $+{var};
                 my $conf_val = $self->cfg->cfg($var);
                 my $op       = $+{op} eq '==' ? 'eq' : 'ne';
-                $matches =
-                  defined($conf_val)
+                $matches = defined($conf_val)
                   && eval "\$self->cfg->cfg(\$var) $op \$val";
             }
             else {
@@ -892,6 +993,42 @@ sub _m_if {
     return $out;
 }
 
+# bpv(MAKE_VAR)
+# Produces prefixed makefile variable name based on MAKE_VAR -> @bp@MAKE_VAR
+sub _m_bpv {
+    my $self = shift;
+    my $var  = shift;
+    $self->is_in_context( backends => 'backend' );
+    return uc( $self->cfg->cfg('backend_prefix') ) . "_" . $var;
+}
+
+# bsv(MAKE_VAR)
+# Produces suffixed makefile variable name based on MAKE_VAR -> MAKE_VAR_@uc(@backend@)@
+sub _m_bsv {
+    my $self = shift;
+    my $var  = shift;
+    $self->is_in_context( backends => 'backend' );
+    return $var . "_" . uc( $self->cfg->cfg('backend') );
+}
+
+# bpm(MAKE_VAR)
+# Produces prefixed makefile macro name based on MAKE_VAR -> $(@bp@MAKE_VAR)
+sub _m_bpm {
+    my $self = shift;
+    my $var  = shift;
+    $self->is_in_context( backends => 'backend' );
+    return '$(' . uc( $self->cfg->cfg('backend_abbr') ) . "_" . $var . ')';
+}
+
+# bsm(MAKE_VAR)
+# Produces suffixed makefile macro name based on MAKE_VAR -> $(MAKE_VAR_@uc(@backend@)@)
+sub _m_bsm {
+    my $self = shift;
+    my $var  = shift;
+    $self->is_in_context( backends => 'backend' );
+    return '$(' . $var . "_" . uc( $self->cfg->cfg('backend') ) . ')';
+}
+
 # varinfo(var1 var2 ...)
 # Dumps information about the context where the variable is defined.
 sub _valstr { defined $_[0] ? $_[0] : '*undef*' }
