@@ -109,7 +109,7 @@ function radixHelper(radix, str, zpos, flags) {
   const letters = radix >= 11 ? lowercase + uppercase + uppercaseTitle + lowercaseTitle : '';
 
   const digitClass = '[\\pN' + letters + ']';
-  const minus = flags & 0x02 ? '(?:-?|\\+?)' : '';
+  const minus = flags & 0x02 ? '(?:-|\\+|−)?' : '';
   const regex = xregexp(
       '^' + minus + digitClass + '(?:_' + digitClass + '|' + digitClass + ')*');
 
@@ -125,6 +125,7 @@ function radixHelper(radix, str, zpos, flags) {
 
   let error;
   number = number.replace(notSimpleDigit, function(match, offset, string) {
+    if (match == '−') return '-';
     const code = match.codePointAt(0);
     const propValueId = numericTypeData.get(code);
 
@@ -628,11 +629,11 @@ function fromJSToReturnValue(ctx, obj) {
 exports.fromJSToReturnValue = fromJSToReturnValue;
 
 function fromJSToArgument(obj) {
-  return fromJS(hll.getHLL('perl6'), obj, false, true);
+  return fromJS(hll.getHLL('Raku'), obj, false, true);
 }
 
 function fromJSToObject(ctx, obj) {
-  return fromJS(hll.getHLL('perl6'), obj, false, false);
+  return fromJS(hll.getHLL('Raku'), obj, false, false);
 }
 
 function fromJS(HLL, obj, isReturnValue, isArgument) {
@@ -681,7 +682,7 @@ function fromJS(HLL, obj, isReturnValue, isArgument) {
   }
 }
 
-function argToJSWithCtx(ctx, obj) {
+/*async*/ function argToJSWithCtx(ctx, obj) {
   if (obj instanceof NativeIntArg || obj instanceof NativeUIntArg) {
     return obj.value;
   } else if (obj instanceof NativeNumArg) {
@@ -689,11 +690,11 @@ function argToJSWithCtx(ctx, obj) {
   } else if (obj instanceof NativeStrArg) {
     return obj.value;
   } else {
-    return toJSWithCtx(ctx, obj);
+    return /*await*/ toJSWithCtx(ctx, /*await*/ obj.$$decont(ctx));
   }
 }
 
-function returnValueToJSWithCtx(ctx, obj) {
+/*async*/ function returnValueToJSWithCtx(ctx, obj) {
   if (obj instanceof NativeNumRet) {
     return obj.value;
   } else if (obj instanceof NativeNumArg) {
@@ -701,25 +702,33 @@ function returnValueToJSWithCtx(ctx, obj) {
   } else if (typeof obj === 'string') {
     return obj;
   } else {
-    return toJSWithCtx(ctx, obj);
+    return /*await*/ toJSWithCtx(ctx, /*await*/ obj.$$decont(obj));
   }
 }
 
-function toJSWithCtx(ctx, obj) {
+/*async*/ function toJSWithCtx(ctx, obj) {
   const HLL = ctx.$$getHLL();
-  if (HLL.get('str_box') && obj.$$istype(ctx, HLL.get('str_box'))) {
+  if (HLL.get('str_box') && /*await*/ obj.$$istype(ctx, HLL.get('str_box'))) {
     return obj.$$getStr();
-  } else if (HLL.get('num_box') && obj.$$istype(ctx, HLL.get('num_box'))) {
+  } else if (HLL.get('num_box') && /*await*/ obj.$$istype(ctx, HLL.get('num_box'))) {
     return obj.$$getNum();
-  } else if (HLL.get('js_box') && obj.$$istype(ctx, HLL.get('js_box'))) {
+  } else if (HLL.get('js_box') && /*await*/ obj.$$istype(ctx, HLL.get('js_box'))) {
     return obj.$$jsObject;
+  } else if (obj === HLL.get('true_value')) {
+    return true;
+  } else if (obj === HLL.get('false_value')) {
+    return false;
+  } else if (obj === HLL.get('null_value')) {
+    return null;
+  } else if (obj.$$getBignum) {
+    return BigInt(obj.$$getBignum().toString());
   } else if (op.isinvokable(obj)) {
-    return function() {
+    return /*async*/ function() {
       const converted = [null, {}];
       for (let i = 0; i < arguments.length; i++) {
         converted.push(fromJSToArgument(arguments[i]));
       }
-      return returnValueToJSWithCtx(ctx, obj.$$apply(converted));
+      return /*await*/ returnValueToJSWithCtx(ctx, /*await*/ obj.$$apply(converted));
     };
   } else if (obj.$$STable === BOOT.StrArray.$$STable) {
     return obj.array;
@@ -942,6 +951,20 @@ class JavaScriptCompiler extends NQPObject {
   }
 };
 
+op.jsruntimerequire = function(ctx, name, prefix) {
+  return fromJSToReturnValue(ctx, require(require.resolve(name, {paths: [prefix]})));
+};
+
+op.jscompiletimerequire = function(ctx, name, prefix) {
+  const required = op.jsruntimerequire(ctx, name, prefix);
+
+  if (required.$$requireStub) {
+    required.$$requireStub(name, prefix);
+  }
+
+  return required;
+};
+
 globalContext.initialize(context => context.compilerRegistry.set('JavaScript', new JavaScriptCompiler()));
 
 class JSBackendStub extends NQPObject {
@@ -993,7 +1016,7 @@ const fakePerl6 = new FakePerl6();
 op.getcomp = function(language) {
   const compilerRegistry = globalContext.context.compilerRegistry;
 
-  if (language === 'perl6' && !compilerRegistry.has(language)) {
+  if (language === 'Raku' && !compilerRegistry.has(language)) {
     return fakePerl6;
   }
 
@@ -2208,8 +2231,8 @@ op.isne_snfg = function(a, b) {
   return (a.normalize('NFC') === b.normalize('NFC')) ? 0 : 1;
 };
 
-op.setjsattr = function(ctx, obj, attr, value) {
-  return obj.$$jsObject[attr] = toJSWithCtx(ctx, value);
+op.setjsattr = /*async*/ function(ctx, obj, attr, value) {
+  return obj.$$jsObject[attr] = /*await*/ toJSWithCtx(ctx, value);
 };
 
 op.getjsattr = function(ctx, obj, attr) {
